using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Collections;
using System.IO;
using System.Runtime.InteropServices;
using FAXCOMEXLib;

namespace URL2Fax
{
    class Program
    {
        [STAThread]
        static int Main(string[] args)
        {
            try
            {
                if (args.Length == 0)
                {
                    throw new Exception("Parameters required.");
                }
                WebPageBitmap WebBitmap = new WebPageBitmap(args[0], 864, 1200, true);
                WebBitmap.Fetch();

                ArrayList Bitmaps = WebBitmap.GetBitmaps(1728, 2400);
                System.Drawing.Imaging.Encoder Encoder1 = System.Drawing.Imaging.Encoder.Compression;
                System.Drawing.Imaging.Encoder Encoder2 = System.Drawing.Imaging.Encoder.SaveFlag;

                EncoderParameters EncoderParameters = new EncoderParameters(2);

                EncoderParameters.Param[0] = new EncoderParameter(Encoder2, (long)EncoderValue.MultiFrame);
                EncoderParameters.Param[1] = new EncoderParameter(Encoder1, (long)EncoderValue.CompressionCCITT4);

                Bitmap Pages;

                Bitmap NewBitmap = new Bitmap(1728,2400);

                string FileName = args[0];
                
                Converter.TranslateImage((Bitmap)Bitmaps[0], ref NewBitmap);
                Pages = NewBitmap;

                FileName = FileName.Replace('\\', '/');
                string[] FileNameParts = args[0].Split('/');
                FileName = System.Environment.GetEnvironmentVariable("TEMP") + "\\" + FileNameParts[FileNameParts.Length - 1] + ".tif";

                Pages.Save(FileName, GetEncoderInfo("image/tiff"), EncoderParameters);

                EncoderParameters.Param[0] = new EncoderParameter(Encoder2, (long)EncoderValue.FrameDimensionPage);

                for (int i = 1; i < Bitmaps.Count; i++)
                {
                    NewBitmap = new Bitmap(1728, 2400);
                    Converter.TranslateImage((Bitmap)Bitmaps[i], ref NewBitmap);
                    Pages.SaveAdd(NewBitmap, EncoderParameters);
                }


                EncoderParameters.Param[0] = new EncoderParameter(Encoder2, (long)EncoderValue.Flush);

                Pages.SaveAdd(EncoderParameters);

                Console.WriteLine(String.Format("Image saved to {0}", FileName));

                if (args.Length > 1)
                {

                    FaxDocument FaxDocument = new FaxDocumentClass();
                    FaxServer FaxServer = new FaxServerClass();

                    FaxDocument.Body = (FileName);

                    FaxDocument.Recipients.Add(args[1], "Fax Recipient");
                    FaxDocument.ReceiptType = 0;

                    FaxDocument.CoverPageType = 0;
                    FaxDocument.ScheduleType = 0;

                    FaxServer.Connect(args[2]);

                    FaxDocument.ConnectedSubmit(FaxServer);
                    FaxServer.Disconnect();

                    Console.WriteLine(String.Format("FAX Scheduled for {0} on {1}", args[1], args[2]));

                }

            }
            catch (Exception ex)
            {
                TextWriter ErrorWriter = Console.Error;
                ErrorWriter.WriteLine(ex.ToString() + System.Environment.NewLine + ex.StackTrace);
                ErrorWriter.WriteLine(@"Usage: URL2FAX.EXE URL [FAXNumber] [FAXServerName]");
                ErrorWriter.WriteLine(@"This program will create a FAX image of the URL and place it in the temp directory.");
                ErrorWriter.WriteLine(@"If you provide FAXNumber and FAXServerName, it will send the TIF to the Windows Fax ");
                ErrorWriter.WriteLine(@"Printer server specified, delivering it to phone number specified.");
                return -1;
            }
            return 0;

        }


        private static ImageCodecInfo GetEncoderInfo(String mimeType)
        {
            int j;
            ImageCodecInfo[] Encoders;
            Encoders = ImageCodecInfo.GetImageEncoders();
            for (j = 0; j < Encoders.Length; ++j)
            {
                if (Encoders[j].MimeType == mimeType)
                    return Encoders[j];
            }
            return null;
        }


    }
    class WebPageBitmap
    {
        private WebBrowser WebBrowser;
        private string Url;
        private int Width;
        private int Height;
        private bool IsReady;


        public WebPageBitmap(string url, int width, int height, bool ScrollBarsEnabled)
        {
            this.Url = url;
            this.Width = width;
            this.Height = height;
            WebBrowser = new WebBrowser();
            WebBrowser.ScriptErrorsSuppressed = true;
            WebBrowser.DocumentCompleted +=
                new WebBrowserDocumentCompletedEventHandler(DocumentCompletedEventHandler);
            WebBrowser.Size = new Size(width, height);
            WebBrowser.ScrollBarsEnabled = ScrollBarsEnabled;
        }

        public void Fetch()
        {
            WebBrowser.Navigate(Url);
            while (WebBrowser.ReadyState != WebBrowserReadyState.Complete)
            {
                Application.DoEvents();
            }

        }
        private void DocumentCompletedEventHandler(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            IsReady = true;
        }

        ~WebPageBitmap()
        {
            /*            webBrowser.Dispose();
             */
        }

        internal ArrayList GetBitmaps(int ThumbWidth, int ThumbHeight)
        {
            ArrayList WebBitmaps = new ArrayList();
            int TotalPages = System.Convert.ToInt32(Math.Ceiling((double)WebBrowser.Document.Body.ScrollRectangle.Height / WebBrowser.Document.Body.ClientRectangle.Height));
            for (int i = 0; i < TotalPages ; i++)
            {
                WebBrowser.Document.Body.ScrollTop = WebBrowser.Document.Body.ClientRectangle.Height * i;
                Bitmap Bitmap = new Bitmap(ThumbWidth, ThumbHeight);
                Rectangle BitmapRect = new Rectangle(0, 0, ThumbWidth, ThumbHeight);
                WebBrowser.DrawToBitmap(Bitmap, BitmapRect);
                Bitmap Thumbnail = new Bitmap(ThumbWidth, ThumbHeight);
                Graphics Gfx = Graphics.FromImage(Thumbnail);
                Gfx.DrawImage(Bitmap, new Rectangle(0, 0, ThumbWidth, ThumbHeight), 0, 0, Width, Height, GraphicsUnit.Pixel);
                Bitmap.Dispose();
                WebBitmaps.Add(Thumbnail);
            }

            return WebBitmaps;
        }
    }
    public static class Converter
    {
        //This code is from Michael McCloskey's  CodeProject.com  article, "Bitonal article Bitonal (TIFF) Image Converter for .NET"
        //Dithering is pseudo-random, hackish
        public static void TranslateImage(Bitmap OriginalBitmap, ref Bitmap MonochromeBitmap)
        {
            Bitmap RgbBitmap = Converter.ConvertToRGB(OriginalBitmap);
            RgbBitmap.SetResolution(196, 204);
            MonochromeBitmap = Converter.ConvertToMonochrome(RgbBitmap);
        }

        public static Bitmap ConvertToRGB(Bitmap OriginalBitmap)
        {
            Bitmap NewImage = new Bitmap(OriginalBitmap.Width, OriginalBitmap.Height, PixelFormat.Format32bppArgb);
            NewImage.SetResolution(OriginalBitmap.HorizontalResolution, OriginalBitmap.VerticalResolution);
            Graphics g = Graphics.FromImage(NewImage);
            g.DrawImageUnscaled(OriginalBitmap, 0, 0);
            g.Dispose();
            return NewImage;
        }

        public static Bitmap ConvertToMonochrome(Bitmap Original)
        {
            Bitmap Source = null;

            // If original bitmap is not already in 32 BPP, ARGB format, then convert
            if (Original.PixelFormat != PixelFormat.Format32bppArgb)
            {
                Source = ConvertToRGB(Original);
            }
            else
            {
                Source = Original;
            }

            // Lock Source bitmap in memory
            BitmapData SourceData = Source.LockBits(new Rectangle(0, 0, Source.Width, Source.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);

            // Copy image data to binary array
            int ImageSize = SourceData.Stride * SourceData.Height;
            byte[] sourceBuffer = new byte[ImageSize];
            Marshal.Copy(SourceData.Scan0, sourceBuffer, 0, ImageSize);

            // Unlock Source bitmap
            Source.UnlockBits(SourceData);

            // Create destination bitmap
            Bitmap Destination = new Bitmap(Source.Width, Source.Height, PixelFormat.Format1bppIndexed);
            Destination.SetResolution(Source.HorizontalResolution, Source.VerticalResolution);

            // Lock destination bitmap in memory
            BitmapData DestinationData = Destination.LockBits(new Rectangle(0, 0, Destination.Width, Destination.Height), ImageLockMode.WriteOnly, PixelFormat.Format1bppIndexed);

            // Create destination buffer
            ImageSize = DestinationData.Stride * DestinationData.Height;
            byte[] DestinationBuffer = new byte[ImageSize];

            int SourceIndex = 0;
            int DestinationIndex = 0;
            int PixelTotal = 0;
            byte DestinationValue = 0;
            int PixelValue = 128;
            int Height = Source.Height;
            int Width = Source.Width;

            bool White1, White2, White3;
            //Threshold 2 , 192 is good for text, 128 is good for images.
            byte Threshold1 = 224;
            byte[] Threshold2 = { 128, 128, 192, 192, 224 };

            bool YMod = true;
            bool XMod = true;
            int Threshold2Index = 0;
            Random RandomIndex = new Random();

            // Iterate lines
            for (int y = 0; y < Height; y++)
            {
                SourceIndex = y * SourceData.Stride;
                DestinationIndex = y * DestinationData.Stride;
                DestinationValue = 0;
                PixelValue = 128;
                YMod = YMod ? false : true;

                // Iterate pixels
                for (int x = 0; x < Width; x++)
                {
                    XMod = XMod ? false : true;
                    Threshold2Index = RandomIndex.Next(4);

                    //is any component, Red, Green or Blue, greater than the 
                    White1 = (sourceBuffer[SourceIndex + 0] & Threshold1) == Threshold1 || (sourceBuffer[SourceIndex + 1] & Threshold1) == Threshold1 || (sourceBuffer[SourceIndex + 0] & Threshold1) == Threshold1;
                    White2 = (sourceBuffer[SourceIndex + 1] & sourceBuffer[SourceIndex + 2] & Threshold2[Threshold2Index]) == Threshold2[Threshold2Index];
                    White3 = ((sourceBuffer[SourceIndex + 0] & sourceBuffer[SourceIndex + 1] & sourceBuffer[SourceIndex + 2]) & Threshold2[Threshold2Index]) == Threshold2[Threshold2Index];
                      

                    if (White1 || (White2 && (XMod || YMod) )|| (White3))
                    {
                        DestinationValue += (byte)PixelValue;
                    }
                    if (PixelValue == 1)
                    {
                        DestinationBuffer[DestinationIndex] = DestinationValue;
                        DestinationIndex++;
                        DestinationValue = 0;
                        PixelValue = 128;
                    }
                    else
                    {
                        PixelValue >>= 1;
                    }
                    SourceIndex += 4;
                }
                if (PixelValue != 128)
                {
                    DestinationBuffer[DestinationIndex] = DestinationValue;
                }
            }

            // Copy binary image data to destination bitmap
            Marshal.Copy(DestinationBuffer, 0, DestinationData.Scan0, ImageSize);

            // Unlock destination bitmap
            Destination.UnlockBits(DestinationData);

            // Return
            return Destination;
        }
    }

}
